==> dobroparser-0.1.1b.tar.gz: DobrochanParser/parser.cpp <==
#include "parser.h"
#include "iostream"
#include <QtNetwork/QtNetwork>
#include <QTextCodec>
#include "tnotify.h"
#include "singleton.h"

parser::parser(QWidget *parent) :
    QWidget(parent),
    url(new QLineEdit()),
    addButton (new QPushButton("Add Thread")),
    deleteButton (new QPushButton("Delete Thread")),
    startButton (new QPushButton("Start")),
    ondemandButton(new QPushButton("On Demand")),
    stopButton (new QPushButton("Stop")),
    testButton (new QPushButton("test")),
    listView (new QTreeWidget()),
    grid (new QGridLayout),
    nm (new QNetworkAccessManager()),
    notifications (&Singleton<TNotify>::Instance()),
    notificationNum(0)
{
    grid->addWidget(addButton, 1,1);
    grid->addWidget(url, 1,3);
    url->setPlaceholderText("Thread URL");
    grid->addWidget(deleteButton, 2,1);
    grid->addWidget(startButton, 3,1);
    grid->addWidget(ondemandButton, 4,1);
    grid->addWidget(stopButton, 5,1);
    //stopButton->setEnabled(false);
    grid->addWidget(testButton, 6,1);
    grid->addWidget(listView, 2,2,6,2);
    listView->setSelectionMode(QAbstractItemView::ExtendedSelection);
    listView->setColumnCount(3);
    QStringList head;
    head << "Name" << "Status" <<"URL";
    listView->setHeaderLabels(head);

    this->setLayout(grid);
    QObject::connect(addButton, SIGNAL(clicked()), this, SLOT(addURL()));
    QObject::connect(url, SIGNAL(returnPressed()), this, SLOT(addURL()));
    QObject::connect(deleteButton, SIGNAL(clicked()), this, SLOT(deleteURL()));
    QObject::connect(this, SIGNAL(requestReplyToThread(TThread*,QNetworkReply*)), this, SLOT(replyToThread(TThread*,QNetworkReply*)));
    QObject::connect(startButton, SIGNAL(clicked()), this, SLOT(start()));
    QObject::connect(stopButton, SIGNAL(clicked()), this, SLOT(stop()));
    QObject::connect(this, SIGNAL(requestListUpdate()), this, SLOT(listUpdate()));
    QObject::connect(testButton, SIGNAL(clicked()), this, SLOT(test()));

}


void parser::addURL(void){
    //QRegExp* regexp = new QRegExp(QString("http://dobrochan.ru/[lor|b]/res/[0-9]*.xhtml"));
    QRegExp* regexp = new QRegExp(QString("http://dobrochan.ru/(mad|lor|b|vg)/res/([0-9]*).xhtml"));
    if(QUrl(url->text()).isValid() and regexp->exactMatch(url->text())){
        urlList.append(new TThread(QUrl(url->text()), this));
        QObject::connect(urlList.last(), SIGNAL(isChanged()), this, SLOT(listUpdate()));
        urlList.last()->setThreadBoard(regexp->cap(1));
        urlList.last()->setThreadNumber(regexp->cap(2));
        //listView->addItem(new QListWidgetItem(url->text()));
        std::cout << urlList.first()->getURL().toString().toStdString() <<std::endl;
        url->clear();
    }else{
        QMessageBox* msgBox = new QMessageBox();
        msgBox->setText("Invalid URL is specified");
        msgBox->show();
    }
}

void parser::deleteURL(void){
    for(int i=listView->topLevelItemCount()-1; i>= 0; i--){
      if(listView->topLevelItem(i)->isSelected()){
          std::cout << "Deleting thread: " << urlList.at(i)->getName().toStdString() << std::endl;
          urlList.removeAt(i);
          emit requestListUpdate();
      }
    }
}

void parser::replyRecieved(QNetworkReply* reply){
    QString data = QString(reply->readAll());
    std::cout << data.toStdString() <<std::endl;
}

void parser::start(void){
    if (!listView->selectedItems().isEmpty()){
        int i = listView->indexOfTopLevelItem(listView->currentItem());

        urlList.at(i)->startWatch();

        /*
        QString mahurl;
        mahurl.append("http://dobrochan.ru/");
        mahurl.append(urlList.at(i)->getThreadBoard());
        mahurl.append("/res/");
        QString threadNumber;
        threadNumber.setNum(urlList.at(i)->getThreadNumber());
        mahurl.append(threadNumber);
        mahurl.append(".json");
        QNetworkRequest request = QNetworkRequest(QUrl(mahurl));
        request.setOriginatingObject(urlList.at(i));
        QNetworkReply *reply = nm->get(request);
        urlList.at(i)->setState(TState::WAITING);

        emit requestReplyToThread(urlList.at(i), reply);
        */
    }
}

void parser::stop(){
    if (!listView->selectedItems().isEmpty()){
        int i = listView->indexOfTopLevelItem(listView->currentItem());

        urlList.at(i)->stopWatch();
    }
}

void parser::listUpdate(void){
    listView->clear();
    if (!urlList.isEmpty()){
        for (int i = 0; i < urlList.count(); i++){
            QStringList added;
            added << urlList.at(i)->getName() << urlList.at(i)->getState() << urlList.at(i)->getURL().toString();
            QTreeWidgetItem* newItem = new QTreeWidgetItem(added);
            newItem->setTextColor(1, urlList.at(i)->getColor());
            listView->addTopLevelItem(newItem);
        }
    }
}

void parser::replyToThread(TThread* mahThread, QNetworkReply* reply){
    QEventLoop loop;
    QObject::connect(reply, SIGNAL(finished()), &loop, SLOT(quit()));
    loop.exec();

    QString data = QString(reply->readAll().constData());

    QString title = getTitleFromReply(data);
    QString message  = getMessageFromReply(data);
    message = toUnicode(message);
    message.truncate(200);
    message.append("<a href='"+mahThread->getURL().toString()+"'> ...--></a>");
    message.insert(0, "<html><body>");
    message.append("</body></html>");
    TNotify* notify = &Singleton<TNotify>::Instance();
    notify->newNotify(title, message);
    //QMessageBox* msgBox = new QMessageBox();
    //msgBox->setText(message);
    //msgBox->show();
    //std::cout << data.toStdString() <<std::endl;
    std::cout << title.toStdString() <<std::endl;
    std::cout << reply->url().toString().toStdString() <<std::endl;

    mahThread->setState(TState::WATCHING);
}

QString parser::getTitleFromReply(QString data){
    QRegExp expTitle("\"title\":\\s\"([^\"]*)\"");
    expTitle.indexIn(data);
    QString title = expTitle.cap(1);
    return toUnicode(expTitle.cap(1));
}

QString parser::getMessageFromReply(QString data){
    QRegExp messageExp("\"message\":\\s\"([^\"]*)\"");
    messageExp.indexIn(data);
    QString tmp(messageExp.cap(1));
    tmp = parser::toUnicode(tmp);
    return tmp;
}


QString parser::toUnicode (QString str){
    QRegExp newLine("\\\\r\\\\n");
    str.replace(newLine, "\n");
    QRegExp uni("\\\\u[0-9a-f]{4}");
    int pos = 0;
    while ((pos = uni.indexIn(str, pos)) != -1) {
        str.replace(pos++, 6, QChar(uni.cap(0).right(4).toUShort(0, 16)));
    }
    return str;
}

void parser::test(void){
    notifications->newNotify("mah parser", "asdfsdf");
}


void parser::addNotify(void){
    notificationNum++;
}

void parser::deleteNotify(void){
    if(notificationNum > 0){
        notificationNum--;
    }else{
        std::cout << "Sorry, but current notification number is zero" << std::endl;
    }
}

==> dobroparser-0.1.2.tar.gz: DobrochanParser/parser.cpp <==
#include "parser.h"
#include "iostream"
#include <QtNetwork/QtNetwork>
#include <QTextCodec>
#include "tnotify.h"
#include "singleton.h"
#include "tthreadlist.h"
#include "tthreadlistitem.h"

parser::parser(QWidget *parent) :
    QWidget(parent),
    url(new QLineEdit()),
    addButton (new QPushButton("Add Thread")),
    deleteButton (new QPushButton("Delete Thread")),
    startAllButton (new QPushButton("Start All")),
    startButton (new QPushButton("Start")),
    ondemandButton(new QPushButton("On Demand")),
    stopButton (new QPushButton("Stop")),
    testButton (new QPushButton("test")),
    listView (new TThreadList()),
    grid (new QGridLayout),
    nm (new QNetworkAccessManager()),
    notifications (&Singleton<TNotify>::Instance()),
    notificationNum(0)
{
    grid->addWidget(addButton, 1,1);
    grid->addWidget(url, 1,3);
    url->setPlaceholderText("Thread URL");
    grid->addWidget(startAllButton, 2,1);
    grid->addWidget(startButton, 3,1);
    grid->addWidget(ondemandButton, 4,1);
    ondemandButton->setDisabled(true);
    grid->addWidget(stopButton, 5,1);
    //stopButton->setEnabled(false);
    grid->addWidget(testButton, 6,1);
    grid->addWidget(deleteButton, 7,1);
    grid->addWidget(listView, 2,2,7,2);
    QSettings settings;
    settings.beginGroup("Urls");
    QStringList saved;
    saved = settings.value("UrlsList").toStringList();
    settings.endGroup();
    for (int i=0; i< saved.count(); i++){
        listView->addThread(QUrl(saved.at(i)));
    }

    this->setLayout(grid);
    QObject::connect(addButton, SIGNAL(clicked()), this, SLOT(addURL()));
    QObject::connect(url, SIGNAL(returnPressed()), this, SLOT(addURL()));
    QObject::connect(deleteButton, SIGNAL(clicked()), this, SLOT(deleteURL()));
    QObject::connect(startButton, SIGNAL(clicked()), this, SLOT(start()));
    QObject::connect(startAllButton, SIGNAL(clicked()), this, SLOT(startAll()));
    QObject::connect(stopButton, SIGNAL(clicked()), this, SLOT(stop()));
    QObject::connect(testButton, SIGNAL(clicked()), this, SLOT(test()));

    this->resize(600,250);

}


void parser::addURL(void){
    //QRegExp* regexp = new QRegExp(QString("(http://dobrochan.ru/[lor|b]/res/[0-9]*.xhtml)"));
    QRegExp* regexp = new QRegExp(QString("(http://dobrochan.ru/(mad|lor|b|vg)/res/([0-9]*).xhtml)"));
    if(QUrl(url->text()).isValid() and regexp->indexIn(url->text()) != -1){
        std::cout<< "Echo 1" << std::endl;
        listView->addThread(QUrl(regexp->cap(1)));
        url->clear();
        std::cout<< "Echo End"<<std::endl;
    }else{
        QMessageBox* msgBox = new QMessageBox();
        msgBox->setText("Invalid URL is specified");
        msgBox->show();
    }
}

void parser::deleteURL(void){
    for(int i=listView->topLevelItemCount()-1; i>= 0; i--){
      if(listView->topLevelItem(i)->isSelected()){
          //std::cout << "Deleting thread: " << urlList.at(i)->getName().toStdString() << std::endl;
          listView->deleteThread(i);
      }
    }
}

void parser::start(void){
    if (!listView->selectedItems().isEmpty()){
        listView->startWatch(listView->currentIndex().row());
    }
}

void parser::startAll(void){
    for(int i=0; i < listView->topLevelItemCount(); i++){
        listView->startWatch(i);
    }
}

void parser::stop(){
    if (!listView->selectedItems().isEmpty()){
        listView->stopWatch(listView->currentIndex().row());
    }
}

QString parser::getTitleFromReply(QString data){
    QRegExp expTitle("\"title\":\\s\"([^\"]*)\"");
    expTitle.indexIn(data);
    QString title = expTitle.cap(1);
    return toUnicode(expTitle.cap(1));
}

QString parser::getMessageFromReply(QString data){
    QRegExp messageExp("\"message\":\\s\"([^\"]*)\"");
    messageExp.indexIn(data);
    QString tmp(messageExp.cap(1));
    tmp = parser::toUnicode(tmp);
    return tmp;
}


QString parser::toUnicode (QString str){
    QRegExp newLine("\\\\r\\\\n");
    str.replace(newLine, "\n");
    QRegExp uni("\\\\u[0-9a-f]{4}");
    int pos = 0;
    while ((pos = uni.indexIn(str, pos)) != -1) {
        str.replace(pos++, 6, QChar(uni.cap(0).right(4).toUShort(0, 16)));
    }
    return str;
}

void parser::test(void){
    notifications->newNotify("mah parser", "asdfsdf");
}


void parser::addNotify(void){
    notificationNum++;
}

void parser::deleteNotify(void){
    if(notificationNum > 0){
        notificationNum--;
    }else{
        std::cout << "Sorry, but current notification number is zero" << std::endl;
    }
}
